using System;
using System.Web;
using System.Security.Principal;
using System.Collections;
using System.Threading;

namespace Demo.ASPNET.MojeTridy
{
	public class MojeApplication : System.Web.HttpApplication 
	{
		public MojeApplication()
		{
			this.AuthorizeRequest+=new EventHandler(PrehodPrincipal);	// zaregistruj handler pro prehozeni principal
			new Thread(new ThreadStart(UklidPrincipalCache)).Start(); // aktivuje periodicky uklid principalCache
		}

		private void PrehodPrincipal(Object sender, EventArgs e)
		{
			IPrincipal user=HttpContext.Current.User;
			if((user==null) || !user.Identity.IsAuthenticated) return;		//anonymni uzivatel, nedelej nic
			string jmeno=user.Identity.Name;
			IntranetPrincipal princ=(IntranetPrincipal) principalCache[jmeno];
			// Pokud neni v cache nebo expiroval ...
			if((princ==null)||(DateTime.Now.Subtract(PrincipalCacheExpirationTime)>princ.Vytvoren))
			{
				princ=IntranetPrincipal.Create(user.Identity);
				principalCache[jmeno]=princ;
			}
			HttpContext.Current.User=(IntranetPrincipal) princ;
			Thread.CurrentPrincipal=princ;
		}

		// Cache slouzi k vyraznemu zvyseni vykonu
		private Hashtable principalCache=new Hashtable();

		// Doba, jak dlouho se drzi Principal objekty v cache
		public TimeSpan PrincipalCacheExpirationTime=TimeSpan.FromSeconds(15); // vychozich 15 sekund

		// Vymazani PrincipalCache (resp. vytvoreni nove)
		public void ClearPrincipalCache()
		{
			principalCache.Clear();
		}

		// Poradek musi byt (Ordnung must sein), kdyby to tu nebylo, hromadilo by se to a utikala by pamet
		private void UklidPrincipalCache()
		{
			while(true)  // Nekonecna smycka
			{
				Thread.Sleep(TimeSpan.FromMinutes(5)); //usni na 5 minut				
				DateTime maximalniDobaVytvoreni=DateTime.Now-PrincipalCacheExpirationTime;
				lock(principalCache)
				{
					string[] klice=new string[principalCache.Count];
					principalCache.Keys.CopyTo(klice,0);
					foreach(string jmeno in klice)
					{
						IntranetPrincipal princ=(IntranetPrincipal) principalCache[jmeno];
						if(princ.Vytvoren<maximalniDobaVytvoreni) principalCache.Remove(jmeno);
					}
				}
			}
		}

	}
}
